import logging

from scrapy.item import Item

from .fields import ListType, DropDocument


logger = logging.getLogger(__name__)


class TypedItem(Item):
    def preprocess(self):
        for field_name, field_type in self.fields.items():
            if not isinstance(field_type, ListType):
                self[field_name] = next(iter(self.get(field_name, [])), None)

            if self.get(field_name) is None and 'default' in field_type:
                self[field_name] = field_type.get('default')

    def validate(self):
        missing = []
        invalid = []

        for field_name, field_type in self.fields.items():
            required = field_type.get('required', False)
            value = self.get(field_name)

            if required and value is None:
                missing.append(field_name)

            if not isinstance(value, field_type):
                invalid.append((field_name, type(value)))

        errors = []

        if missing:
            errors.append("Missing fields: %s" % ', '.join(missing))

        if invalid:
            fields = ("%s (%r expected, got %s)" % (
                self[f], self.fields[f], t.__name__) for f, t in invalid)
            errors.append("Invalid fields: %s" % ', '.join(fields))

        if errors:
            raise DropDocument(*errors)
