import logging

from gargantext.datasource import Scraper, RISResponse
from gargantext.datasource.items import Document, DocumentLoader
from gargantext.utils.dates import to_datetime, datetime


__all__ = ['RISScraper']


logger = logging.getLogger(__name__)


class RISDocumentLoader(DocumentLoader):
    def parse(self, entry):
        if not (entry.get('id') or '').strip():
            entry['id'] = entry.get('doi')

        if not (entry.get('title') or '').strip():
            entry['title'] = entry.get('primary_title')

        if not (entry.get('source') or '').strip():
            entry['source'] = entry.get('secondary_title')

        # In Scopus PY may refer to the first day of the publication period
        # with the year omitted, in this case we have to get year from Y2
        # FIXME Sometimes month is also omitted, how to be aware of this?
        publication = to_datetime(entry.get('publication'),
                default=datetime(1, 1, 1))
        if publication.year == 1:
            secondary = entry.get('secondary_publication')
            if secondary:
                secondary = to_datetime(secondary)
                entry['publication'] = publication.replace(year=secondary.year)

        self.add_values({
            k: v for k, v in entry.items() if k in Document.fields
        })


class RISScraper(Scraper):
    name = 'ris'
    expects = RISResponse

    def parse(self, response):
        for entry in response.parse():
            yield RISDocumentLoader(entry).load()
            if self.reach_limit():
                return
